/* Simple data plot program */
/*----------------------------------------------------------------------------
 *  Copyright (c) 1991, 1992, 1993, 1994   Ohio University
 *                                         Accelerator Laboratory
 *
 *  This software was partially developed under a United States Government
 *  license described in the Notice file included as part of this distribution
 * 
 *    Ohio University Accelerator Laboratory, Athens, Ohio  45701
 * 
 *    carter@oual3.phy.ohiou.edu   Tel: (614) 593-1984  Fax: (614) 593-1436 
 *    odonnell@oual3.phy.ohiou.edu Tel: (614) 593-1977  Fax: (614) 593-1436 
 *----------------------------------------------------------------------------
 *
 * Authors:
 *
 *	Don Carter
 *      John O'Donnell
 *
 *	Ohio University Accelerator Laboratory
 *
 * Original version DAQ created 1991
 * Revision History:
 *
 *----------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <varargs.h>

#include <signal.h>

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "oual.h"
  
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <Xm/Label.h>

#include <Xm/MainW.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrolledW.h>
#include <Xm/Xm.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>

void Ex1CB();
void Ex2CB();
void Ex3CB();
void Ex4CB();
void ZoomCB();
void UnZoomCB();
void PanlCB();
void PanrCB();
void YupCB();
void YdnCB();
void AutoCB();
void HomeCB();
void RefCB();
void QuitCB();
void RedrawCB();
void ResizeCB();
void xs_wprintf();
void TimeOutCB();
int  readdata();
int  updatedata();
void start_rubber_band();
void end_rubber_band();
void track_rubber_band();
void mouse_motion();

GC	SimpleGC;
Pixmap pixmap;

int CBcount = 0;
int id;
int parent;
int rtype;
int halt_update=0;
int current_x=0,current_y=0,mouse_channel,mouse_value;
Dimension new_width, new_height;

String colors[] = {
   "Black", "Grey", "Red", "Brown", "Purple", "Green", "Blue", 
   "Magenta", "Turquoise", "Cyan", "Wheat", "Violet", "LimeGreen",
   "Yellow", "Pink", "White" 
};  

struct dataplot {
	int	data[OUAL_MAX_DEFINE_LENGTH];
	int	disk,tag,start,stop,max,min,istart,istop,autoy;
	int     parent;
	Widget	info;
} plot;

typedef struct {
        int start_x, start_y, last_x, last_y;
        GC gc;
} rubber_band_data;

/* struct plotbuffer {
	unsigned start;
	unsigned stop;
	unsigned long savdat;
	unsigned long savtim;
	int inuse;
	unsigned data[OUAL_MAX_DEFINE_LENGTH];
	} oual_plot;
*/
char *shmpltptr;
struct plotbuffer *plotbufferptr;

	Widget  canvas;

main(argc, argv)
	int argc;
	char *argv[];
{
	Widget 	toplevel, frame, infobar, quitbutton, ex1, ex2, ex3, ex4, home;
	Widget  zoom, unzoom, panl, panr, yauto, yup, ydn, ref;
        rubber_band_data data;
	XtAppContext app;
	XSetWindowAttributes	attrs;
	XmString	xmstr;
	Arg	wargs[10];
	extern GC	SimpleGC;
	XGCValues	gcvalues;
	int	n;
	unsigned long	valuemask;

	toplevel = XtVaAppInitialize(&app, "Simple", NULL, 0, 
				&argc, argv, NULL, NULL);

	/* XtInitialize is called first to let to digest any command
		line arguments for windows; it will leave anything it
		doesn't want for us. */
//      printf("\nEntering fpdcrefx...");
/*
	printf("Enter disk and tag number to plot:");
	scanf("%d %d",&plot.disk,&plot.tag);
*/
	id = atoi(argv[2]);
	parent = atoi(argv[3]);
	if (readdata(plot.disk,plot.tag,&plot) != 0) {
		printf("readdata failed\n");
		exit(0);
	}
/*
	printf("Readdata completed, disk %d tag %d start %d stop %d\n",
			plot.disk,plot.tag,plot.start,plot.stop);

*/
	n = 0;
	XtSetArg(wargs[n], XmNheight, 500); n++;
	XtSetArg(wargs[n], XmNwidth, 700); n++;
	frame = XtCreateManagedWidget("Frame", xmFormWidgetClass,
				toplevel, wargs, n);

	n = 0;
	XtSetArg(wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(wargs[n], XmNorientation, XmHORIZONTAL); n++;
	infobar = XmCreateRowColumn(frame, "infobar", wargs, n);
	XtManageChild(infobar);

	n = 0;
	xmstr = XmStringCreate("Quit", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
	XtSetArg(wargs[n], XmNadjustLast, False); n++;
	XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
	quitbutton = XmCreatePushButton(infobar, "quitbutton", wargs, n);
	XtAddCallback(quitbutton, XmNarmCallback, QuitCB, 1);
	XtAddCallback(quitbutton, XmNactivateCallback, QuitCB, 2);
	XtAddCallback(quitbutton, XmNdisarmCallback, QuitCB, 3);
	XtManageChild(quitbutton);

        n = 0;
        xmstr = XmStringCreate("Ex1", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        ex1 = XmCreatePushButton(infobar, "ex1", wargs, n);
        XtAddCallback(ex1, XmNarmCallback, Ex1CB, 1);
        XtAddCallback(ex1, XmNactivateCallback, Ex1CB, 2);
        XtAddCallback(ex1, XmNdisarmCallback, Ex1CB, 3);
        XtManageChild(ex1);

        n = 0;
        xmstr = XmStringCreate("Ex2", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        ex2 = XmCreatePushButton(infobar, "ex2", wargs, n);
        XtAddCallback(ex2, XmNarmCallback, Ex2CB, 1);
        XtAddCallback(ex2, XmNactivateCallback, Ex2CB, 2);
        XtAddCallback(ex2, XmNdisarmCallback, Ex2CB, 3);
        XtManageChild(ex2);

        n = 0;
        xmstr = XmStringCreate("Ex3", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        ex3 = XmCreatePushButton(infobar, "ex3", wargs, n);
        XtAddCallback(ex3, XmNarmCallback, Ex3CB, 1);
        XtAddCallback(ex3, XmNactivateCallback, Ex3CB, 2);
        XtAddCallback(ex3, XmNdisarmCallback, Ex3CB, 3);
        XtManageChild(ex3);

        n = 0;
        xmstr = XmStringCreate("Ex4", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        ex4 = XmCreatePushButton(infobar, "ex4", wargs, n);
        XtAddCallback(ex4, XmNarmCallback, Ex4CB, 1);
        XtAddCallback(ex4, XmNactivateCallback, Ex4CB, 2);
        XtAddCallback(ex4, XmNdisarmCallback, Ex4CB, 3);
        XtManageChild(ex4);

        n = 0;
        xmstr = XmStringCreate("Zoom", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        zoom = XmCreatePushButton(infobar, "zoom", wargs, n);
        XtAddCallback(zoom, XmNarmCallback, ZoomCB, 1);
        XtAddCallback(zoom, XmNactivateCallback, ZoomCB, 2);
        XtAddCallback(zoom, XmNdisarmCallback, ZoomCB, 3);
        XtManageChild(zoom);

        n = 0;
        xmstr = XmStringCreate("UnZoom", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        unzoom = XmCreatePushButton(infobar, "unzoom", wargs, n);
        XtAddCallback(unzoom, XmNarmCallback, UnZoomCB, 1);
        XtAddCallback(unzoom, XmNactivateCallback, UnZoomCB, 2);
        XtAddCallback(unzoom, XmNdisarmCallback, UnZoomCB, 3);
        XtManageChild(unzoom);

        n = 0;
        xmstr = XmStringCreate("PanL", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        panl = XmCreatePushButton(infobar, "panl", wargs, n);
        XtAddCallback(panl, XmNarmCallback, PanlCB, 1);
        XtAddCallback(panl, XmNactivateCallback, PanlCB, 2);
        XtAddCallback(panl, XmNdisarmCallback, PanlCB, 3);
        XtManageChild(panl);

        n = 0;
        xmstr = XmStringCreate("PanR", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        panr = XmCreatePushButton(infobar, "panr", wargs, n);
        XtAddCallback(panr, XmNarmCallback, PanrCB, 1);
        XtAddCallback(panr, XmNactivateCallback, PanrCB, 2);
        XtAddCallback(panr, XmNdisarmCallback, PanrCB, 3);
        XtManageChild(panr);

        n = 0;
        xmstr = XmStringCreate("Auto", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        yauto = XmCreatePushButton(infobar, "yauto", wargs, n);
        XtAddCallback(yauto, XmNarmCallback, AutoCB, 1);
        XtAddCallback(yauto, XmNactivateCallback, AutoCB, 2);
        XtAddCallback(yauto, XmNdisarmCallback, AutoCB, 3);
        XtManageChild(yauto);

        n = 0;
        xmstr = XmStringCreate("Yup", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        yup = XmCreatePushButton(infobar, "yup", wargs, n);
        XtAddCallback(yup, XmNarmCallback, YupCB, 1);
        XtAddCallback(yup, XmNactivateCallback, YupCB, 2);
        XtAddCallback(yup, XmNdisarmCallback, YupCB, 3);
        XtManageChild(yup);
                            
        n = 0;
        xmstr = XmStringCreate("Ydn", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        ydn = XmCreatePushButton(infobar, "ydn", wargs, n);
        XtAddCallback(ydn, XmNarmCallback, YdnCB, 1);
        XtAddCallback(ydn, XmNactivateCallback, YdnCB, 2);
        XtAddCallback(ydn, XmNdisarmCallback, YdnCB, 3);
        XtManageChild(ydn);

        n = 0;
        xmstr = XmStringCreate("Home", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        home = XmCreatePushButton(infobar, "home", wargs, n);
        XtAddCallback(home, XmNarmCallback, HomeCB, 1);
        XtAddCallback(home, XmNactivateCallback, HomeCB, 2);
        XtAddCallback(home, XmNdisarmCallback, HomeCB, 3);
        XtManageChild(home);

        n = 0;
        xmstr = XmStringCreate("Ref", XmSTRING_DEFAULT_CHARSET);
        XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
        XtSetArg(wargs[n], XmNadjustLast, False); n++;
        XtSetArg(wargs[n], XmNpacking, XmPACK_NONE); n++;
        ref = XmCreatePushButton(infobar, "ref", wargs, n);
        XtAddCallback(ref, XmNarmCallback, RefCB, 1);
        XtAddCallback(ref, XmNactivateCallback, RefCB, 2);
        XtAddCallback(ref, XmNdisarmCallback, RefCB, 3);
        XtManageChild(ref);
           
                            
	n = 0;
	xmstr = XmStringCreate("Label", XmSTRING_DEFAULT_CHARSET);
	XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
	plot.info = XmCreateLabel(infobar, "info", wargs, n);
	XtManageChild(plot.info);

	n = 0;
	XtSetArg(wargs[n], XmNheight, 420); n++;
	XtSetArg(wargs[n], XmNwidth, 580); n++;
	XtSetArg(wargs[n], XmNx, 0); n++;
	XtSetArg(wargs[n], XmNy, 50); n++;
	XtSetArg(wargs[n], XmNborderWidth, 1); n++;
	XtSetArg(wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
	XtSetArg(wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
	XtSetArg(wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	XtSetArg(wargs[n], XmNtopWidget, infobar); n++;
	canvas = XmCreateDrawingArea(frame, "canvas", wargs, n);
	XtAddCallback(canvas, XmNresizeCallback, ResizeCB, 1);
        XtAddCallback(canvas, XmNexposeCallback, RedrawCB, "plot");

        XtAddEventHandler(canvas, ButtonPressMask, FALSE,
                          start_rubber_band, &data);
        XtAddEventHandler(canvas, ButtonMotionMask, FALSE, 
                          track_rubber_band, &data);
        XtAddEventHandler(canvas, ButtonReleaseMask, FALSE, 
                          end_rubber_band, &data);
        XtAddEventHandler(canvas, PointerMotionMask, FALSE,
                          mouse_motion, &data);

	XtManageChild(canvas);

	xs_wprintf(plot.info," %s", argv[1]);

	XtRealizeWidget(toplevel);

        XGrabButton(XtDisplay(canvas), AnyButton, AnyModifier,
                    XtWindow(canvas), TRUE,
                    ButtonPressMask | ButtonMotionMask |
                    ButtonReleaseMask,
                    GrabModeAsync, GrabModeAsync,
                    XtWindow(canvas),
                    XCreateFontCursor(XtDisplay(canvas),
                                      XC_crosshair));

        data.gc = xs_create_xor_gc(canvas);

	XtAppAddTimeOut(XtWidgetToApplicationContext(toplevel),1000,TimeOutCB,toplevel);

	gcvalues.foreground = BlackPixel(XtDisplay(canvas), 0);
	gcvalues.background = WhitePixel(XtDisplay(canvas), 0);
	SimpleGC = XCreateGC(XtDisplay(canvas), XtWindow(canvas),
				(GCForeground | GCBackground), &gcvalues);

	attrs.bit_gravity = ForgetGravity;
	valuemask = CWBitGravity;
	XChangeWindowAttributes( XtDisplay(canvas), XtWindow(canvas),
				valuemask, &attrs);

	XtAppMainLoop(app);

	exit(0);
}

GC xs_create_xor_gc(w)
    Widget        w;
{
XGCValues values;
GC    gc;
Arg   wargs[10];
XtSetArg(wargs[0], XtNforeground, &values.foreground);
XtSetArg(wargs[1], XtNbackground, &values.background);
XtGetValues(w, wargs,2);
values.foreground = values.foreground ^ values.background;
values.line_style = LineSolid; //LineOnOffDash
values.function = GXxor;
gc = XtGetGC(w,GCForeground | GCBackground |
             GCFunction | GCLineStyle, &values);
return gc;
}

void start_rubber_band(w,data,event)
Widget    w;
rubber_band_data  *data;
XEvent            *event;
{
halt_update=1;
data->last_x = data->start_x = event->xbutton.x;
data->last_y = data->start_y = event->xbutton.y;
XDrawLine(XtDisplay(w), XtWindow(w), data->gc, data->start_x,
          data->start_y, data->last_x, data->last_y);
}
void track_rubber_band(w,data,event)
Widget    w;
rubber_band_data  *data;
XEvent            *event;
{
current_x = event->xbutton.x;
current_y = event->xbutton.y;
if(halt_update == 1)
  {
  XDrawLine(XtDisplay(w), XtWindow(w), data->gc, data->start_x,
          data->start_y, data->last_x, data->last_y);
  data->last_x = event->xbutton.x;
  data->last_y = event->xbutton.y;
  XDrawLine(XtDisplay(w), XtWindow(w), data->gc, data->start_x,
          data->start_y, data->last_x, data->last_y);
  }
}

void end_rubber_band(w,data,event)
Widget    w;
rubber_band_data  *data;
XEvent            *event;
{
int start,stop,len,j;
XDrawLine(XtDisplay(w), XtWindow(w), data->gc, data->start_x,
          data->start_y, data->last_x, data->last_y);
data->last_x = event->xbutton.x;
data->last_y = event->xbutton.y;
start=plot.start;
len=plot.stop-plot.start+1;
if(event->xbutton.button == Button1)
  plot.start = start + (len*((float)data->start_x/(float)new_width));
if(event->xbutton.button == Button3)
  plot.stop = start + (len*((float)data->start_x/(float)new_width));
stop = start + (len*((float)data->last_x/(float)new_width));
if(stop > (plot.start+1)) plot.stop=stop;
if(event->xbutton.button == Button2){
  plot.start=plot.istart;
  plot.stop=plot.istop;
  }
plot.autoy=1;
plot.max=16;
for (j = plot.start; j <= plot.stop; j++) {
    while (plot.data[j] > plot.max) plot.max = plot.max << 1;
    }
halt_update=0;
}
 
void mouse_motion(w,data,event)
Widget    w;
rubber_band_data  *data;
XEvent            *event;
{
current_x = event->xbutton.x;
current_y = event->xbutton.y;
}

void QuitCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
//      --
//      struct dataplot *client_data;
//      XmDrawingAreaCallbackStruct *call_data;
{
        XColor  col, unused;
        Colormap cmap = DefaultColormapOfScreen (XtScreen (w));

        extern GC       SimpleGC;
        Display *dsply;
        Window drawable;
/*
        dsply = XtDisplay(w);
        drawable = XtWindow(w);
        --
{
*/
        static int flag;
        int i;
        i = client_data;

        /* This is done to assure all the activate callbacks are complete
                before the program ends.  Refer to the X Window System
                Programming and Applications book by D.A. Young, pg 77. */

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */

                        if(shmctl(id,IPC_RMID,0) == -1)
                        {
                        printf("\nshmctl release failed in QUIT.\n");
                        }
                        XAllocNamedColor (dsply,cmap,"Black",&col,&unused);
                        XSetForeground(dsply,SimpleGC,col.pixel);
                        XtCloseDisplay(XtDisplay(w));
                        exit(0);

                }

        }
}

void Ex1CB(w, client_data, call_data)
	Widget	w;
	int	*client_data;
	XmAnyCallbackStruct	*call_data;
{
	static int flag;
	int i;
	i = client_data;

	/* This is done to assure all the activate callbacks are complete
		before the program ends.  Refer to the X Window System
		Programming and Applications book by D.A. Young, pg 77. */

	switch(i) {

	case 1:	/* Arm */
			
		flag = 0;	/* if we disarm now, nothing happens */
		break;

	case 2: /* Activate */

		flag = 1;	/* user released button, stand by to quit */
		break;

	case 3: /* Disarm */

		if (flag == 1) {	/* finished with things, now exit */

		  //if(rtype == 0) 
                    plot.stop = plot.start + (((plot.stop-plot.start)+1)/4)-1;
                  if(XtIsRealized(canvas))
                  //if(rtype == 1) rtype = 2;
                  //else if(rtype == 2) rtype = 1; 
                  XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
		}

	}
}
void Ex2CB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i, delta;
        i = client_data;

        /* This is done to assure all the activate callbacks are complete
                before the program ends.  Refer to the X Window System
                Programming and Applications book by D.A. Young, pg 77. */

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */

                  delta = (((plot.stop-plot.start)+1) / 4);
		  plot.start = plot.start + delta;
	          plot.stop = plot.start + delta - 1;
                  if(XtIsRealized(canvas))
                  XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}
void Ex3CB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i, delta;
        i = client_data;

        /* This is done to assure all the activate callbacks are complete
                before the program ends.  Refer to the X Window System
                Programming and Applications book by D.A. Young, pg 77. */

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */


                  delta = (((plot.stop-plot.start)+1) / 4);
                  plot.start = plot.start + (2 * delta);
                  plot.stop = plot.start + delta - 1;
                  if(XtIsRealized(canvas))
                  XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}
void Ex4CB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i, delta;
        i = client_data;

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */

                  delta = (((plot.stop - plot.start)+1) / 4);
                  plot.start = plot.start + (3 * delta);
                  plot.stop = plot.start + delta - 1;
                  if(XtIsRealized(canvas))
                  XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}

void ZoomCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
	int delta;
        i = client_data;

        switch(i) {

        case 1: /* Arm */
                flag = 0;       /* if we disarm now, nothing happens */
                break;
        case 2: /* Activate */
                flag = 1;       /* user released button, stand by to quit */
                break;
        case 3: /* Disarm */
                if (flag == 1) {        /* finished with things, now exit */
                   if(rtype == 0) delta = (plot.stop - plot.start ) / 4;
		   else delta = 64;
                   plot.start = plot.start + delta;
                   if((plot.stop - delta) > plot.start)plot.stop = plot.stop - delta;
                   if(XtIsRealized(canvas))
                   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}

void UnZoomCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
        int delta;
        i = client_data;

        switch(i) {

        case 1: /* Arm */
                flag = 0;       /* if we disarm now, nothing happens */
                break;
        case 2: /* Activate */
                flag = 1;       /* user released button, stand by to quit */
                break;
        case 3: /* Disarm */
                if (flag == 1) {        /* finished with things, now exit */
                   if(rtype == 0) delta = (plot.stop - plot.start ) / 4;
                   else delta = 64;
                   plot.start = plot.start - delta;
                   plot.stop = plot.stop + delta;
		   //if(plot.start < 0) plot.start = 0;
		   if(plot.start < plot.istart) plot.start = plot.istart;
                   if(plot.stop > plot.istop) plot.stop = plot.istop;
                   if(XtIsRealized(canvas))
                   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}
  

void PanlCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
        int delta;
        i = client_data;

        switch(i) {

        case 1: /* Arm */
                flag = 0;       /* if we disarm now, nothing happens */
                break;
        case 2: /* Activate */
                flag = 1;       /* user released button, stand by to quit */
                break;
        case 3: /* Disarm */
                if (flag == 1) {        /* finished with things, now exit */
                   if(rtype == 0) delta = (plot.stop - plot.start ) / 10;
                   else delta = 64;
                   plot.start = plot.start - delta;
		   //if (plot.start < 0) plot.start = 0;
                   if (plot.start < plot.istart) plot.start = plot.istart;
                   plot.stop = plot.stop - delta;
		   if (plot.stop < 0) plot.stop = 10;
                   if(XtIsRealized(canvas))
                   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}

void PanrCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
        int delta;
        i = client_data;

        switch(i) {

        case 1: /* Arm */
                flag = 0;       /* if we disarm now, nothing happens */
                break;
        case 2: /* Activate */
                flag = 1;       /* user released button, stand by to quit */
                break;
        case 3: /* Disarm */
                if (flag == 1) {        /* finished with things, now exit */
                   if(rtype == 0) delta = (plot.stop - plot.start ) / 10;
                   else delta = 64;
                   plot.start = plot.start + delta;
                   if (plot.start > plot.istop) plot.start = plot.istop - 10;
                   plot.stop = plot.stop + delta;
                   if (plot.stop < plot.start) plot.stop = plot.istop;
		   if (plot.stop > plot.istop) plot.stop = plot.istop;
                   if(XtIsRealized(canvas))
                   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}

void AutoCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
        i = client_data;

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */
                   plot.autoy = 1; 
                   if(XtIsRealized(canvas))
                   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}

void YupCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
        i = client_data;

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */
                   plot.autoy = 0;
                   plot.max = plot.max * 2;
                   if(XtIsRealized(canvas))
                   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}

void YdnCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
        i = client_data;

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */
                   plot.autoy = 0;
                   plot.max = plot.max / 2;
                   if(XtIsRealized(canvas))
                   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}

void HomeCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
        i = client_data;

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */

                   plot.autoy = 1;
                   //plot.start = 0;
                   plot.start = plot.istart;
                   plot.stop = plot.istop;
                   if(XtIsRealized(canvas))
		   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}
  
void RefCB(w, client_data, call_data)
        Widget  w;
        int     *client_data;
        XmAnyCallbackStruct     *call_data;
{
        static int flag;
        int i;
        i = client_data;

        switch(i) {

        case 1: /* Arm */

                flag = 0;       /* if we disarm now, nothing happens */
                break;

        case 2: /* Activate */

                flag = 1;       /* user released button, stand by to quit */
                break;

        case 3: /* Disarm */

                if (flag == 1) {        /* finished with things, now exit */

                   if(rtype == 1) rtype = 2;
                   else if(rtype == 2) rtype = 1;
                   //else if(rtype == 3) rtype = 1;
		   updatedata(&plot);
                   if(XtIsRealized(canvas))
		   XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
                }

        }
}


void TimeOutCB(client_data, call_data )
	Widget client_data;
	XtIntervalId *call_data;
{
	int time,j;
 	updatedata(&plot);
        if(halt_update == 0)
          {
          if(XtIsRealized(canvas))
           XClearArea(XtDisplay(canvas),XtWindow(canvas),0,0,0,0,TRUE);
          }

     if(plot.autoy == 1) {
        plot.max = 16;
        for (j = plot.start; j <= plot.stop; j++) {
                while (plot.data[j] > plot.max) plot.max = plot.max << 1;
                }
        }

	time = plot.stop - plot.start;
        if (time > 1024) time = 1024;
	if (time <  512) time = 250;
        if (rtype == 1) time = 1.5*(float)time;
	XtAppAddTimeOut(XtWidgetToApplicationContext(client_data),time,TimeOutCB,client_data);
} 

void RedrawCB( w, client_data, call_data )
	Widget	w;
	struct dataplot	*client_data;
	XmDrawingAreaCallbackStruct *call_data;
{
	XColor  col, unused;
        Colormap cmap = DefaultColormapOfScreen (XtScreen (w));

	extern GC	SimpleGC;
	int x, y, oldx, oldy, i, j, sum, width, z;
        int rx,ry,rb,rk;
	float xstep, ystep;
        int curmouse_x,curmouse_y,ix,iy,iz,thresh;
	char strbuff[100];
	Display *dsply;
	Window drawable;

	dsply = XtDisplay(w);
	drawable = XtWindow(w);

	i = plot.max;
/*	xs_wprintf(plot.info, "%d", i); */ 

	/* Get the window size */
	XtVaGetValues(w,
		XmNwidth, &new_width,
		XmNheight, &new_height,
		NULL);


     if(plot.autoy == 1) {
	plot.max = 16;
        for (j = plot.start; j <= plot.stop; j++) {
                while (plot.data[j] > plot.max) plot.max = plot.max << 1;
                }
        }                  

	xstep = new_width;
	xstep = xstep / (plot.stop - plot.start + 1);
	ystep = new_height - 20;
	ystep = ystep / plot.max; 

/*	printf("Redraw: xstep = %f ystep = %f\n",xstep,ystep); */
/*	printf("start = %d   stop = %d   max = %d\n", plot.start, plot.stop, plot.max); */
	sum = 0;
	oldx = 0;   
	oldy = new_height - 20;
        i = 0; 
	if(rtype == 0)
            {
            //XAllocNamedColor (dsply,cmap,"Black",&col,&unused);
            //XSetForeground(dsply,SimpleGC,col.pixel);
            for (j = plot.start; j <= plot.stop; j++) 
		{
                sum += plot.data[j];
                if(j == mouse_channel)mouse_value=plot.data[j];
		y = (new_height - 20) - (plot.data[j] * ystep);
                x = i * xstep;
                if(j != mouse_channel)
                    {
                    XDrawLine(dsply,drawable,SimpleGC,x,y-1,x,y+1);
		    XDrawLine(dsply,drawable,SimpleGC,x-1,y,x+1,y);
                    }
                else
                    {
                    XAllocNamedColor (dsply,cmap,"Magenta",&col,&unused);
                    XSetForeground(dsply,SimpleGC,col.pixel);
                    XDrawLine(dsply,drawable,SimpleGC,x,y-5,x,y+5);
                    XDrawLine(dsply,drawable,SimpleGC,x-5,y,x+5,y); 
                    XAllocNamedColor (dsply,cmap,"Black",&col,&unused);
                    XSetForeground(dsply,SimpleGC,col.pixel);
                    }
                i++;
                oldx = x;
                oldy = y;
                }
            }
        if((rtype == 1) || (rtype == 3))
            {
            sum = 0;
            i = 0;
            x=(float)new_width*0.038;
            y=(float)new_height*0.03;
            xstep=(float)x*1.7;
            for (j=1; j<16; j++)
                {
                XAllocNamedColor (dsply,cmap,colors[j],&col,&unused);
                XSetForeground(dsply,SimpleGC,col.pixel);
                XFillRectangle(dsply,drawable,SimpleGC,x,y,new_width/20,new_height/20);
                sprintf(strbuff,"%d",plot.max*j/16);
                XAllocNamedColor (dsply,cmap,"Black",&col,&unused);
                XSetForeground(dsply,SimpleGC,col.pixel);
                XDrawString(dsply,drawable,SimpleGC,x+1,y+(new_height/20)-2,strbuff,strlen(strbuff));
                x += xstep;
                }
            xstep = ((float)new_width*0.9)/64.0;
            ystep = ((float)new_height*0.8)/64.0;
            curmouse_x=((((float)current_x-(float)new_width*0.05)/
                       ((float)new_width*0.9))*64.0);
            curmouse_y=64.0-(((float)current_y-(ystep/2.0)-((float)new_height*0.1))/ystep);
            XAllocNamedColor (dsply,cmap,"White",&col,&unused);
            XSetForeground(dsply,SimpleGC,col.pixel);
            XDrawRectangle(dsply,drawable,SimpleGC,
                          (float)new_width*0.05,(float)new_height*0.1+ystep,
                          64*xstep,64*ystep);
            XAllocNamedColor (dsply,cmap,"Black",&col,&unused);
            for (j = plot.start; j <= plot.stop; j++) 
                { 
                sum += plot.data[j];
                iy = j/64;
                ix = j-(64*iy);
                //x = ((j-(64*(j/64))) * xstep)+(float)new_width*0.05;
                x = (ix * xstep)+(float)new_width*0.05;
                y = (((float)new_height*0.9) - ((j/64) * ystep));
                if ((ix==curmouse_x) && (iy==curmouse_y))
                      mouse_value=plot.data[j];
                z=((float)plot.data[j] / (float)plot.max)*15.0;
                if(z > 15) z = 15;
                if(z > 0)
                    {
                    XAllocNamedColor (dsply,cmap,colors[z],&col,&unused);
                    XSetForeground(dsply,SimpleGC,col.pixel);

                    XFillRectangle(dsply,drawable,SimpleGC,x,y,xstep*0.8,ystep*0.8);
                    if ((ix==curmouse_x) && (iy==curmouse_y))
                       {
                       XAllocNamedColor (dsply,cmap,"White",&col,&unused);
                       XSetForeground(dsply,SimpleGC,col.pixel);
                       XDrawRectangle(dsply,drawable,SimpleGC,
                            x-(0.1*xstep), y-(0.1*ystep),xstep,ystep);
                       }
                    XAllocNamedColor (dsply,cmap,"Black",&col,&unused);
                    XSetForeground(dsply,SimpleGC,col.pixel);
                    }
                i++;
		oldx = x;
		oldy = y;
		}
            XAllocNamedColor (dsply,cmap,"White",&col,&unused);
            XSetForeground(dsply,SimpleGC,col.pixel);
            }
        if((rtype == 2) || (rtype == 3))
            {
            sum = 0;
            oldx = 0;  
            oldy = new_height - 20;
            ystep = new_height - 20;
            ystep = ystep / plot.max;
            i = 0;
            //xstep=xstep/2.0;
            thresh=((float)plot.max*0.01)+1;
            ystep=ystep/2.0;
            XAllocNamedColor (dsply,cmap,"White",&col,&unused);
            XSetForeground(dsply,SimpleGC,col.pixel);
            xstep = ((float)new_width*0.9)/128.0;
            //ystep = ((float)new_height*0.8)/64.0;
            for (j = plot.start; j <= plot.stop; j++)
                {
                iy = j/64;
                ix = j-(64*iy);
                iz = plot.data[j];
                //x = ((j-(64*(j/64))) * xstep)+(float)new_width*0.05;
                x = ((ix * xstep)+(float)new_width*0.05)+((float)iy*xstep);
                //y = ((((float)new_height*0.9) - ((j/64) * ystep)))-((float)iy*ystep);
                
                
                sum += iz;
                //if(j == mouse_channel)mouse_value=plot.data[j];
                y = (new_height - 40) - (iz * ystep)-
                    (((float)(new_height-40)/128.0)*iy);
                if(iz == 0) XDrawPoint(dsply,drawable,SimpleGC,x,y);
                else if(iz < thresh) XDrawLine(dsply,drawable,SimpleGC,x,y,x+1,y);
                else 
                  {
                  XDrawLine(dsply,drawable,SimpleGC,x,y-1,x,y+1);
                  XDrawLine(dsply,drawable,SimpleGC,x-1,y,x+1,y);
                  if(ix != 0)XDrawLine(dsply,drawable,SimpleGC,oldx,oldy,x,y);
                  }
                //i++;
                oldx = x;
                oldy = y;
                }
            }
  mouse_channel = (((float)current_x/(float)new_width)*((plot.stop-plot.start)+1))+plot.start;
  if(rtype == 0)
    sprintf(strbuff,
    "Start = %d   MaxCounts = %d   Cursor(chan,value) = (%d,%d)   SumCounts = %d   Stop = %d",
      plot.start,plot.max,mouse_channel,mouse_value,sum,plot.stop); 
  else
    sprintf(strbuff,
    "Ystart = %d   MaxCounts = %d   Cursor(x,y,counts) = (%d,%d,%d)  SumCounts = %d   YStop = %d",
      plot.start/64,plot.max,curmouse_x,curmouse_y,mouse_value,sum,plot.stop/64);

  
  XDrawString(dsply,drawable,SimpleGC,10,new_height-5,strbuff,strlen(strbuff));
  XDrawLine(dsply,drawable,SimpleGC,0,new_height-20,new_width,new_height-20);
  XDrawLine(dsply,drawable,SimpleGC,new_width/2,new_height-18,new_width/2,
      new_height-22);
  XAllocNamedColor (dsply,cmap,"Black",&col,&unused);
  XSetForeground(dsply,SimpleGC,col.pixel);

}

void set_color(w, client_data, call_data)
   	Widget w;
        XtPointer client_data;
        XtPointer call_data;
{
   String color = (String) client_data;
   Display *dpy = XtDisplay (w);
   Colormap cmap = DefaultColormapOfScreen (XtScreen (w));
   XColor  col, unused;
   /*
   if(!XAllocNamedColor (dpy,cmap,color,&col,&unused)) {
      char buf[32];
      sprintf (buf, "Can't alloc %s", color);
      XtWarning (buf);
      return;
   }
   */
   XSetForeground (dpy,SimpleGC,col.pixel);
}

void ResizeCB( w, client_data, call_data )
	Widget	w;
	int	*client_data;
	XmDrawingAreaCallbackStruct *call_data;
{

/*	printf("ResizeCB() called %d, client_data = %d, reason=%d\n",
			CBcount, client_data,call_data->reason);
*/
	CBcount++;

}

int readdata(disk,tag,plot)
	int	disk,tag;
	struct dataplot *plot;
{
	char 	fname[128];
	int	rdisk,rtag,rstart,rstop,rdate,rtime;
	int	i,j,datum;
/*
	printf("Reading data for disk %d tag %d", disk,tag);
  
	sprintf(fname,"data/plotbuffer");

	if ((input = fopen(fname,"r")) == NULL) {
		perror("data file open failed");
		return(-1);
	}

	fscanf(input,"%d %d %d %d %d %d",&rdisk,&rtag,&rstart,&rstop,
					&rdate,&rtime);
  
	printf("readdata: disk %d tag %d start %d stop %d date %d time %d\n",
			rdisk, rtag, rstart, rstop, rdate, rtime);
*/
	printf("readdata shmatt id = %d. \n",id);
	if((int) (shmpltptr = (char *) shmat(id,0,0)) == -1)
		{
		printf("\nshmatt failed in readdata.\n");
		exit(0);
		}
	plotbufferptr = (struct plotbuffer *) shmpltptr;
	rdate = plotbufferptr->savdat;
	rtime = plotbufferptr->savtim;
	rstart = plotbufferptr->start;
	rstop = plotbufferptr->stop;
        rtype = plotbufferptr->type;
        if(rtype == 5) rtype = 0;
	
        i = (int)rstart;
//	i = 0;
/* 	fscanf(input,"%8d",&datum);
	plot->min = datum;
*/
	datum=plotbufferptr->data[i];
	plot->min = 0;
	plot->max = datum;
	plot->data[i++] = datum;
	for ( j = rstart+1; j <= rstop; j++) {
		datum = plotbufferptr->data[i];    
		plot->data[i++] = datum;
/*		if (datum < plot->min) plot->min = datum;
*/
		if (datum > plot->max) plot->max = datum;
	}
	plot->start = rstart;
        plot->istart = rstart;
	plot->stop = rstop;
        plot->istop = rstop;
/*
	plot->start = 0;
	plot->istart = 0;
	plot->stop = i - 1;
	plot->istop = i - 1;
*/
	plot->autoy = 1;
/*	printf("Maximum counts %d\n",plot->max); */
/*      shmpltptr->inuse = 0; */
	plotbufferptr->inuse = 0;
	if(shmdt(shmpltptr) == -1)
	{
	printf("\nshmdt failed in readdata.\n");
	}	
/*	if(shmctl(id,IPC_RMID,0) == -1)
	{
	printf("\nshmctl release failed in readdata.\n");
	}
*/
	return(0);
}


int updatedata(plot)
        struct dataplot *plot;
{
        int     rdisk,rtag,rstart,rstop,rdate,rtime;
        int     i,j,datum;
/*	printf("signalling process %d to refresh\n",parent); */
	kill(parent,SIGUSR2);
        if((int) (shmpltptr = (char *) shmat(id,0,0)) == -1)
                {
                printf("\nshmatt failed in updatedata. id = %d.\n",id);
                exit(0);
                }
        plotbufferptr = (struct plotbuffer *) shmpltptr;
        rstart = plotbufferptr->start;
        rstop = plotbufferptr->stop;
        i = 0;
        datum=plotbufferptr->data[i];
        plot->min = 0;
        /* plot->max = datum; */
        plot->data[i++] = datum;
        for ( j = rstart+1; j <= rstop; j++) {
                datum = plotbufferptr->data[i];
                plot->data[i++] = datum;
                /* if (datum > plot->max) plot->max = datum; */
        }
        if(shmdt(shmpltptr) == -1)
        {
        printf("\nshmdt failed in updatedata.\n");
        }
/*      if(shmctl(id,IPC_RMID,0) == -1)
        {
        printf("\nshmctl release failed in updatedata.\n");
        }
*/

        return(0);
}
  
                             
/* Taken from Young, pg 141. */
void xs_wprintf(va_alist)
	va_dcl
{
	Widget	w;
	char	*format;
	va_list	args;
	char	str[512];
	Arg	wargs[10];
	XmString	xmstr;
	int	n;

	va_start(args);

	w = va_arg(args, Widget);	/* get the widget to write to */
	if (!XtIsSubclass( w, xmLabelWidgetClass)) {
		XtError("xs_wprintf() requires a Label Widget");
	}

	format = va_arg(args, char *);

	vsprintf(str, format, args);
	xmstr = XmStringLtoRCreate(str, XmSTRING_DEFAULT_CHARSET);

	n = 0;
	XtSetArg(wargs[n], XmNlabelString, xmstr); n++;
	XtSetValues( w, wargs, n);

	va_end(args);
}

// #include "graphics.h"
/*  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    R I N P T X   --  wait/check for input reports


    fw = 0  -->  no wait, only check for report
         1  -->  wait on report

    fm = 0  -->  do not consider mouse motion reports
         1  -->         consider mouse motion reports

    fb = 0  -->  do not consider mouse button reports
         1  -->         consider mouse button reports

    fk = 0  -->  do not consider keyboard reports
         1  -->         consider keyboard reports

    x  <--  current x-pixel location of mouse cursor
    y  <--  current y-pixel location of mouse cursor

    b = -1  <--  no     mouse button was pushed
         1  <--  left   mouse button was pushed
         2  <--  middle mouse button was pushed
         3  <--  right  mouse button was pushed

    k = -1  <--  no keyboard key was pushed
      >= 0  <--  keysym of keyboard key that was pushed

    rinptx = 0  <--  requested report did not occur
             1  <--  requested report did occur

*/
int rinptx (fw,fm,fb,fk,x,y,b,k)
int fw,fm,fb,fk,*x,*y,*b,*k;

{
        Display *dsply;
        XEvent report;
	int keycode,index;

        dsply = XtDisplay(canvas);
        *x = *y = *b = *k = -1;
       
        while (1)
        {
                if(fw == 0 && XPending(dsply) == 0) return(0);

                XNextEvent(dsply,&report);

                switch (report.type)
                {
                        case MotionNotify:

                                if(fm == 0) break;

                                *x = report.xmotion.x;
                                *y = report.xmotion.y;

                                return(1);
                   
                        case ButtonPress:

                                if(fb == 0) break;

                                *x = report.xbutton.x;
                                *y = report.xbutton.y;
                                *b = report.xbutton.button;

                                return(1);
                   
                        case KeyPress:

                                if(fk == 0) break;

                                *x      = report.xkey.x;
                                *y      = report.xkey.y;
                                keycode = report.xkey.keycode;
                                index   = report.xkey.state;
                                *k      = XKeycodeToKeysym(dsply,keycode,
                                                index);

                                return(1);
                   
                }
        }
}
void rflshx ()
{
        Display *dsply;
	dsply = XtDisplay(canvas);
        XFlush(dsply);
}
