
/*----------------------------------------------------------------------------
 *  Copyright (c) 1991-2001                Ohio University
 *                                         Accelerator Laboratory
 *
 *  This software was partially developed under a United States Government
 *  license described in the Notice file included as part of this distribution
 *
 *    Ohio University Accelerator Laboratory, Athens, Ohio  45701
 *
 *    carter@oual3.phy.ohiou.edu   Tel: (614) 593-1984  Fax: (614) 593-1436
 *    odonnell@oual3.phy.ohiou.edu Tel: (614) 593-1977  Fax: (614) 593-1436
 *----------------------------------------------------------------------------
 *
 * Authors:
 *
 *	Don Carter
 *      John O'Donnell
 *
 *	Ohio University Accelerator Laboratory
 *
 * Original version DAQ created 1991
 * Revision History:
 *
 *----------------------------------------------------------------------------*/

/* Main routine for the PC data acquisition system */

#include <sys/time.h>
#include <sys/resource.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
//#include <process.h>
#include "alloc.h"
#include "conio.h"
#include "dos.h"
//#include <graphics.h>
#include "graphics.h"

/* following added to support shared memory  d.carter*/

#include <sys/types.h> 
#include <sys/ipc.h>
#include <sys/shm.h> 

/* do not include OUAL.H */

#define OUAL_MAX_DEFINES 150
#define OUAL_MAX_DEFINE_LENGTH 1048576 
#define OUAL_MAX_NAME_LENGTH 16
#define OUAL_MAX_VECTORS 150

#define OUAL_SHMSIZE sizeof(struct databuffer)
#define OUAL_SHMPLOTSIZE sizeof(struct plotbuffer)
#define OUAL_BUFFERSIZE 10240

typedef enum {T_WORD, T_BAR, T_AMP, T_SEMI, T_GT, T_GTGT, T_LT,
			  T_NL, T_EOF, T_INTEGER, T_FLOAT} TOKEN;

struct databuffer {
	char dav;
        unsigned nevents;
	unsigned device[OUAL_BUFFERSIZE];
	unsigned channel[OUAL_BUFFERSIZE];
	unsigned data[OUAL_BUFFERSIZE];
	unsigned abort;
	} oual_buff;

struct plotbuffer {
	unsigned start;
	unsigned stop;
	unsigned long savdat;
	unsigned long savtim;
        int type;
	int inuse;
	unsigned data[OUAL_MAX_DEFINE_LENGTH];
	} oual_plot;

typedef struct  rump_header{

	char filename[80];                  // Filename
	char date[80];                      // Date spectrum collected
	char ltct[80];                      // Live time/Clock time information
	char id[80];                        // Identifier string
	float e0;                           // Incident energy            (MeV)
	int   zbeam;                        // Atomic Z of incident
	float mbeam;                        // Atomic mass of incident    (amu)
	int   cbeam;                        // Charge state of beam
	float q;                            // Total accumulated charge    (uC)
	float current;                      // Average beam current        (nA)
	float scale1,scale2;                // Conversion MCA chan # -> keV
	float first;                        // Channel number of first data pt
	float fwhm;                         // Detector resolution        (keV)
	int   geom;                         // Geometry identifier
	float phi,theta,psi;                // Scattering angles      (degrees)
	float omega;                        // Detector solid angle       (mSr)
	float corr;                         // Random correction factor


};

struct {
	unsigned type;
	char nbytes;
	unsigned *address;
	long length;
	char name[OUAL_MAX_NAME_LENGTH];
	unsigned conversion_gain;
	unsigned shift;
	unsigned digital_offset;
	unsigned lld;
	unsigned uld;
/* The next three lines added July 93 - ceb */
	float slope;
	float intcept;
	int ndat;
/* next two lines added for prescale option  d.e.carter */
	int prescale_init;
	int prescale;
/* next line added for RUMP file parameters j.c.Keay */
	struct rump_header * rump_define;

} oual_define[OUAL_MAX_DEFINES];

//extern unsigned _stklen = 8000; /* set stack size if not 4096 */

extern char oual_system_name[];

int oual_vector[OUAL_MAX_VECTORS];
char oual_gated_vector[OUAL_MAX_VECTORS];
int oual_nrgates, oual_maxrgates=150;

unsigned oual_gate[20],oual_rgate[900];
unsigned oual_ngates = 10, oual_t0 = 4095;
unsigned oual_verbose = 0;
int oual_prescale = 10, oual_prescale_init = 10;


int oual_ndefines = 0;
int oual_nvectors = 0;
int oual_disk     = 0;
int oual_tag      = 0;
int oual_file_index = -1;
int oual_retcode  = 0; // return code from last daq command that returns codes
int oual_sofint   = 0; // indicates current interrupt level
int oual_intmask  = 1; // indicates whether software interrupts are allowed
                       // 0 = interrupts allowed, 1 = masked, 
int oual_plot_flag = 0; // indicates to return to plot after interrupt
int oual_terminate = 0; // indicates that preset acquire counts satisfied
int oual_cmnd_nmbr = 0; // modify debug line counter for &goto correction
unsigned oual_counter  = 0; // current preset acquire counter
long oual_return1;          // function return value
unsigned oual_rawdata, oual_adcnumber, oual_adcdata, oual_router, oual_anal = 0;

/* Common command line buffer */

char oual_cmnd[133];				/* last token read by get_token() */
int oual_last_token;				/* last type returned by gettoken() */

#define OUAL_MAX_CMND_LEN 132

FILE *oual_iunit;
FILE *oual_ounit;

int oual_nest_level=0,oual_ioperm=0;
char oual_nest_name[9];

/* global definition for shared memory pointer */

int pid;
char *shmptr;
char *shmpltptr;
struct databuffer *databufferptr;
struct plotbuffer *plotbufferptr;
int adc8_int(); 
int refresh();
int id;

main(argc,argv)
	int argc;
	char *argv[];
{
	int i;
        FILE *f;
        char string[80];
        unsigned data;
/*        int id;    */

/*
*	for (i=0; i < argc; i++) {
*
*		printf("Argument %d: '%s'\n",i,argv[i]);
*
*	}
*	printf("All aguments are ignored for now...\n");
*/

	for (i = 0; i < OUAL_MAX_VECTORS; i++) oual_vector[i] = -1;
	for (i = 0; i < OUAL_MAX_VECTORS; i++) oual_gated_vector[i] = 0;
	for (i = 0; i < 900             ; i++) oual_rgate[i] = 0;
	oual_nest_level = 0;


	textmode(3);
	textbackground(1);
	textcolor(15);
// linux	textattr(0x1f);
	clrscr();

	//if(registerfarbgidriver(EGAVGA_driver_far) < 0) exit(1);
        //if(registerbgidriver(EGAVGA_driver) < 0) exit(1);
	
	printf("\n\n%s",oual_system_name);
	printf("\nX11_SVGA driver registered");
        printf("\nShared Memory Version. Size = %d",OUAL_SHMSIZE);
        printf("\nPC Data Acquisition System (Linux)\n  January 12, 2001\n");

        /* get shared memory segment of OUAL_SHMSIZE bytes */
	if((id = shmget (IPC_PRIVATE, OUAL_SHMSIZE, 0644)) == -1) 
	{
		printf("\nshmget failed. exiting\n");
		exit(1);
	}
	else
	{
	printf("\nShared memory id = %d",id);
	}

/* install interrupt handler for DAQ */

	signal(SIGUSR1,adc8_int);

/* install interrupt handler for refrsh */

	signal(SIGUSR2,refresh);

/* fork child process to process data into data buffer in shared memory */

	if ((pid = fork()) == -1)
	{
		printf("\n fork failed\n");
		exit(1);
	}

/* child DAQ process */

	if (pid == 0)
	{
		if ((int) (shmptr = (char *) shmat(id,0,0)) == -1)
		{
			printf("\n child shmatt failed\n");
			fflush(stdout);
                        exit(1);
		}
		else
		{
			printf("\n child shmptr = %x",shmptr);
		}
		
		printf("\nChild pid = %d. priority = %d",getpid(),getpriority(0,getpid()));
		databufferptr = (struct databuffer *) shmptr;

/* initialize databuffer */

		databufferptr->dav = 0; /* set no data available */
		databufferptr->nevents = 0;
                databufferptr->abort = 0; /* not time to abort */
		databufferptr->data[0] = 1;
		databufferptr->channel[0] = 1;
		databufferptr->device[0] = 0;
		databufferptr->dav = 0; /* set data available */

                if (ioperm(0,1023,1) == 0) 
                     {
                     printf("\n IOperm Succeeded FOR CHILD to 1023\n");
                     setpriority(PRIO_PROCESS,0,-4);
                     }
                else {
                     printf("\n IOperm Rejected FOR CHILD (not root)");
                     databufferptr->dav = 1; /* trick so that dav never occurs for no IOperm */
                     }
 
                /*      Open daq device to process data    */
                if ((f = fopen ("/dev/daq8dev","r")) == NULL) 
                  {
                  printf("\n Open of /dev/daq8dev failed.");
                  goto child_abort;
                  }
                  
                fflush(stdout);
                while (databufferptr->abort == 0)
                    {
                    if(databufferptr->dav == 0) 
                        {
                          data=(fgetc(f)&0xff)|((fgetc(f)&0xff)<<8);
                          //printf("\n%04x  ",data);
                          while (databufferptr->dav == 1) {}; /* if parent is dumping buffer, wait */
                          if(databufferptr->nevents < (OUAL_BUFFERSIZE - 10))
                            databufferptr->data[databufferptr->nevents++]=data;
                          //if((databufferptr->nevents > 100)) /* && ((data & 0x6000) == 0x0)) */
                          if((databufferptr->nevents == 100))  
                            {
                            //databufferptr->dav = 1;
                            kill(getppid(),SIGUSR1);
                            }
                        }
                    }
                fclose(f);
           child_abort:
                printf("\nChild Aborting");        
/* acknowledge the request to abort with abort = 2 */

                databufferptr->abort = 2;  

/* detach the shared memory segment from the process data space */

                if (shmdt(shmptr) == -1)
                {
                        printf("\nCHILD shmdt detach shared memory failed");
                }
                else
                {
                        printf("\nCHILD shared memory detached");
                }

                printf("\nCHILD exiting\n");
                fflush(stdout);
                exit(0);

	}

/* parent DAQ Process */

	if (pid > 0)
	{
		if ((int) (shmptr = (char *) shmat(id,0,0)) == -1)
		{
			printf("\n parent DAQ shmatt failed\n");
			exit(1);
		}
		else
		{
		 	printf("\n parent DAQ shmptr = %x",shmptr);
		}

        	databufferptr = (struct databuffer *) shmptr;
		databufferptr->dav = 0; 
		if (ioperm(0,1023,1) == 0) 
                     {
                     printf("\n IOperm Succeeded FOR PARENT to 1023");
                     setpriority(PRIO_PROCESS,0,-4);
                     oual_ioperm=1;
                     }
                else
                     {
                     printf("\n IOperm Rejected FOR PARENT (not root)");
                     }
                printf("\n Parent pid = %d. priority = %d",getpid(),getpriority(0,getpid()));
                                
                delay(2000);
                
                term_init();
                
                if(oual_ioperm == 0) printf("\n\nWARNING: I/O operations not permitted (not root)");

                printf("\n\nDAQ>");
                fflush(stdout);
        
                command_proc(stdin,stdout); /* start the command interpreter  */
		
                exit(0); 
	}
}







        
