
/*----------------------------------------------------------------------------
 *  Copyright (c) 1991, 1992, 1993, 1994   Ohio University
 *                                         Accelerator Laboratory
 *
 *  This software was partially developed under a United States Government
 *  license described in the Notice file included as part of this distribution
 * 
 *    Ohio University Accelerator Laboratory, Athens, Ohio  45701
 * 
 *    carter@oual3.phy.ohiou.edu   Tel: (614) 593-1984  Fax: (614) 593-1436 
 *    odonnell@oual3.phy.ohiou.edu Tel: (614) 593-1977  Fax: (614) 593-1436 
 *----------------------------------------------------------------------------
 *
 * Authors:
 *
 *	Don Carter
 *      John O'Donnell
 *
 *	Ohio University Accelerator Laboratory
 *
 * Original version DAQ created 1991
 * Revision History:
 *  17-aug-1994  modified to pass plotted data in shared mem
 *               segment instead of writing to disk.  /dec
 *
 *----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <oual.h>
#include <time.h>

/* shared memory support */

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>

int REF_id;
int REF_index;

#define OUAL_SHMPLOTSIZE sizeof(struct plotbuffer)

int fpdc()

{

int id;
int chanctr,i,index;
unsigned long savtim,savdat;
struct tm *tblock;
time_t now;
char fname[16],fpath[80];
char option[16],optnmbr[16];
char strbuff[20];
int disk,tag,start,stop;
int tagincr;
int rumpflag;
int pid;
unsigned *ptr;

//printf("\nENTERING xplot.c");

if (gettoken(fname) != T_WORD) {
	fprintf(oual_ounit,"\nERROR(xplot): Need file name to plot.");
	flush();
	return(1);
	}

index = def_index(fname);
if (index == -1) {
	fprintf(oual_ounit,"\nERROR(xplot): File %s not defined.",fname);
	flush();
	return(2);
	}

disk = 0;
tag = 0;
start = 0;
stop = oual_define[index].length - 1;

while((i = gettoken(option)) != T_NL) {	 /* more tokens, maybe options */
	if (i != T_WORD) {					/* if not a word then not a */
		fprintf(oual_ounit,				/* valid option, either...*/
			"\nERROR(xplot): Bad option '%s'\n",option);
		return(3);
	}
	else if (strcmp(option,"start") == 0) {	/* set start channel */
		if (gettoken(optnmbr) != T_INTEGER) {	/* bad option parameter */
			fprintf(oual_ounit,
				"\nERROR(xplot): Bad start channel '%s'\n",optnmbr);
			return(4);
		} else {
			start = atoi(optnmbr);
		}
	} 
	else if (strcmp(option,"stop") == 0) {	/* set stop channel */
		if (gettoken(optnmbr) != T_INTEGER) {	/* bad option parameter */
			fprintf(oual_ounit,
				"\nERROR(xplot): Bad stop channel '%s'\n",optnmbr);
			return(5);
		} else {
			stop = atoi(optnmbr);
		}
	} 
	else {
		fprintf(oual_ounit,"\nERROR(xplot): Bad option '%s'\n",option);
		return(6);
	}
}

//printf("\nXplot getting sharred memory...");

if((id=shmget (IPC_PRIVATE, OUAL_SHMPLOTSIZE, 0644)) == -1)
  {
  printf("\nshmget failed in xplot\n");
  flush();
  return(4);
  }
if ((int) (shmpltptr = (char *) shmat(id,0,0)) == -1)
  {
  printf("\nshmatt failed in xplot\n");
  flush();
  return(4);
  }

REF_index = index;
REF_id = id;

plotbufferptr = (struct plotbuffer *) shmpltptr;

now = time(NULL);
tblock = localtime(&now);
 
plotbufferptr->savtim = 10000*(unsigned long)(tblock->tm_hour) + 
			100*(unsigned long)(tblock->tm_min) + 
			(unsigned long)tblock->tm_sec;
plotbufferptr->savdat = 10000*(unsigned long)(tblock->tm_year) + 
			100*(unsigned long)(tblock->tm_mon) + 100 + /*for month=0*/ 
			(unsigned long)tblock->tm_mday;
chanctr = 0;
ptr = oual_define[index].address;
ptr += start;
for (i=start; i<=stop; i++) plotbufferptr->data[i] = *ptr++;
plotbufferptr->start = start;
plotbufferptr->stop = stop;
plotbufferptr->inuse = 1;
plotbufferptr->type = oual_define[index].type;
flush();
if((pid=fork()) == -1) {
  printf("\nFork Failed in pdc\n");
  return;
  }
if(pid == 0){  /* child plot process */
//printf("\n Child fpdc fork executing fpdcrefx...");
  if(plotbufferptr->type == 1)sprintf(strbuff,"./daqxplot %s %d %d -bg black -fg white\n",fname,id,getppid());
  else sprintf(strbuff,"./daqxplot %s %d %d -bg white -fg black\n",fname,id,getppid());
  system(strbuff);
  //system(sprintf(strbuff,"./fpdcrefx %s %d %d",fname,id,getppid()));
  //printf("\n...%s...",strbuff);
  //printf("\n system(fpdcrefx fname id ppid executed...");
  exit(0);
  }
else 
  {
  while(plotbufferptr->inuse == 1) {}  /* wait till done */
  if (shmdt(shmpltptr) == -1)
    {
    printf("\nshmdt failed in fpdc. \n");
    }
/*  if (shmctl(id,IPC_RMID,0) == -1)
    {
    printf("\nshmctl release failed in fpdc.\n");
    }
*/
  return(0);
  }
}
