#ifndef __GRAPH_H__
#define __GRAPH_H__ 

enum Colors { 
    black, blue, green, cyan, red, magent, brown, lightgray, darkgray, 
    lightblue, lightgreen, lightcyan, lightred, lightmagenta, yellow, white
};

enum WriteModes { 
    copyput, xorput, orput, andput, notput };
enum LineStyles { 
    solidln, dottedln, centerln, dashedln, userbitln
};
enum FillStyles { 
    emptyfill, solidfill, linefill, ltslashfill, slashfill, bkslashfill,
    ltbkslashfill, hatchfill, xhatchfill, interleavefill, widedotfill, 
    closedotfill, userfill
};
enum TextDirections { 
    horizdir, vertdir 
};
enum FontTypes { 
    defaultfont, triplexfont, smallfont, sansseriffont, gothicfont 
};

#define normwidth					1
#define thickwidth					3

#define dottedlinelength				2
#define centrelinelength				4

#define usercharsize					0
#define maxcolors					15

#define lefttext					0
#define centertext					1
#define righttext					2
#define bottomtext					0
#define toptext						2

#define clipon                                          1
#define clipoff                                         0

#define topon                                           1
#define topoff                                          0

enum GraphicsErrors {
	grok = 0,
	grnoinitgraph = -1,
	grnotdetected = -2,
	grfilenotfound = -3,
	grinvaliddriver	= -4,
	grnoloadmem = -5,
	grnoscanmem = -6,
	grnofloodmem = -7,
	grfontnotfound = -8,
	grnofontmem = -9,
	grinvalidmode =	-10,
	grerror = -11,
	grioerror = -12,
	grinvalidfont = -13,
	grinvalidfontnum = -14,
	grinvaliddevicenum = -15,
	grinvalidversion = -18
};


/* graphics drivers constants, includes x11 which is particular to xbgi. */
#define detect						0
#define cga						1
#define mcga						2
#define ega						3
#define ega64						4
#define egamono						5
#define ibm8514						6
#define hercmono					7
#define att400						8
#define vga						9
#define pc3270						10

/* graphics modes constants. */
#define cgac0						0
#define cgac1						1
#define cgac2						2
#define cgac3						3
#define cgahi						4

#define mcgac0						0
#define mcgac1						1
#define mcgac2						2
#define mcgac3						3
#define mcgamed						4
#define mcgahi						5

#define egalo						0
#define egahi						1

#define ega64lo						0
#define ega64hi						1

#define egamonohi					3

#define hercmonohi					0

#define att400c0					0
#define att400c1					1
#define att400c2					2
#define att400c3					3
#define att400med					4
#define att400hi					5

#define vgalo						0
#define vgamed						1
#define vgahi						2
#define pc3270hi   				        0
#define ibm8514lo  					0
#define ibm8514hi  					1

typedef struct {
    int x;
    int y;
    int xstart;
    int ystart;
    int xend;
    int yend;
} arccoordstype;

struct fillpatterntype { 
    char pattern[8];
};

struct pointtype { 
    int x;
    int y;
};

struct fillsettingstype {
    int pattern;
    int color;
};

struct linesettingstype {
    int linestyle;
    unsigned int upattern;
    int thickness;
};

struct palettetype {
    unsigned char size;
    signed char colors[16];
};

struct textsettingstype {
    int font;	
    int direction;  
    int charsize;
    int horiz;
    int vert;
};

struct viewporttype {
    int x1;
    int y1;
    int x2;
    int y2;
    int clip;
};

extern int bgiemu_handle_redraw;

extern "C" {
void arc(int, int, int, int, int);
void bar(int, int, int, int);
void bar3d(int, int, int, int, int, int);
void circle(int, int, int);
void cleardevice(void);
void clearviewport(void);
void closegraph(void);
void detectgraph(int&, int&);
void drawpoly(int, pointtype*);
void ellipse(int, int, int, int, int, int);
void fillellipse(int, int, int, int);
void fillpoly(int, pointtype*);
void floodfill(int, int, int);
void getarccoords(arccoordstype&);
void getaspectratio(word&, word&);
int getbkcolor(void);
int getcolor(void);
palettetype* getdefaultpalette();
char* getdrivername(void);
void getfillpattern(fillpatterntype&);
void getlinesettings(linesettingstype&);
void getfillsettings(fillsettingstype&);
int getgraphmode(void);
void getimage(int, int, int, int, void*);
int getmaxcolor(void);
int getmaxmode(void);
int getmaxx(void);
int getmaxy(void);
char* getmodename(int);
void getpalette(palettetype&);
int getpalettesize(void);
unsigned int getpixel(int, int);
void gettextsettings(textsettingstype&);
void getviewsettings(viewporttype&);
int getx(void);
int gety(void);
void graphdefaults(void);
char* grapherrormsg(int);
int graphresult(void);
unsigned int imagesize(int, int, int, int);
void initgraph(int* device, int* mode, char const*);
void line(int, int, int, int);
void linerel(int, int);
void lineto(int, int);
void moverel(int, int);
void moveto(int, int);
void outtext(char const*);
void outtextxy(int, int, char const*);
void pieslice(int, int, int, int, int);
void putimage(int, int, void*, int);
void putpixel(int, int, int);
void rectangle(int, int, int, int);
void restorecrtmode(void);
void sector(int, int, int, int, int, int);
void setactivepage(int);
void setallpalette(palettetype const&);
void setaspectratio(int, int);
void setbkcolor(int);
void setcolor(int);
void setfillpattern(fillpatterntype const&, int);
void setfillstyle(int, int);
void setgraphmode(int);
void setlinestyle(int, unsigned int, int);
void setpalette(int, int);
void setrgbpalette(int, int, int, int);
void settextjustify(int, int);
void settextstyle(int, int, int);
void setusercharsize(int, int, int, int);
void setviewport(int, int, int, int, int);
void setvisualpage(int);
void setwritemode(int);
int textheight(char const*);
int textwidth(char const*);
int getch(void);
int kbhit(void);		
void delay(int msec);

};

inline void getdefaultpalette(palettetype& pal) { 
    pal = *getdefaultpalette(); 
}
inline int  readkey() { 
    return getch(); 
}
inline int  keypressed() { 
    return kbhit(); 
}
inline void outtext(char ch) { 
    char str[2] = {ch, '\0'}; 
    outtext(str); 
}
inline void outtextxy(int x, int y, char ch) { 
    char str[2] = {ch, '\0'}; 
    outtextxy(x, y, str); 
}
inline int textheight(char ch) { 
    char str[2] = {ch, '\0'}; 
    return textheight(str);
}
inline int textwidth(char ch) {
    char str[2] = {ch, '\0'}; 
    return textwidth(str);
}

inline void initgraph(int& device, int& mode, char const* driver, 
		      bool handle_redraw = true)
{
    bgiemu_handle_redraw = handle_redraw;
    initgraph(&device, &mode, driver);
}


inline void restorecrtmode() {}


#endif






