/*----------------------------------------------------------------------------
 *  Copyright (c) 1991, 1992, 1993, 1994, 1995   Ohio University
 *                                               Accelerator Laboratory
 *
 *  This software was partially developed under a United States Government
 *  license described in the Notice file included as part of this distribution
 *
 *    Ohio University Accelerator Laboratory, Athens, Ohio  45701
 *    carter@oual3.phy.ohiou.edu   Tel: (614) 593-1984  Fax: (614) 593-1436
 *----------------------------------------------------------------------------
 * Author: 	Don Carter
 * Facility:    Ohio University Accelerator Laboratory
 *
 * Original version DAQ created 1991
 *
 * replay for Ray Wheeler  16-april-97, modified 9-14-98 by RTW 
 *----------------------------------------------------------------------------*/
/* --------------------------- the setup ---------------------------------------
route =  0 : unrouted spectra
route =  1 : detector 1
route =  2 : detector 2
route = 10 : pileup spectra

vector 10*(route) + 0 = energy			e
vector 10*(route) + 1 = tof			tof
vector 10*(route) + 2 = e vs tof		e_tof
vector 10*(route) + 3 = e3			e3
vector 10*(route) + 4 = e3 vs e			e3_e
vector 10*(route) + 5 = (e3 + e) / 2		e3e
vector 10*(route) + 6 = e3 vs (e3 + e)/2	e3_e3e
vector 10*(route) + 7 = mass			mass
vector 10*(route) + 8 = m_e                   	m_e
vector 10*(route) + 9 = m_tof                   m_tof 

vector 90             = event energy 			(adc0 = routed energy)
vector 91             = event tof			(adc1 = routed tof)
vector 92             = event e3			(adc2 = e3)
vector 93             = event route
vector 98             = bci 				(adc 4)
vector 99             = dead time 			(adc 5)
vector 109            = router histogram for pileup			      
------------------------------------------------------------------------------*/

#include <dos.h>
#include <bios.h>
#include <stdio.h>
#include <oual.h>
#include <stdlib.h>
#include <string.h>
#include <alloc.h>
#include <time.h>

//extern unsigned oual_rgate[], oual_nrgates, oual_t0;

int rayrep() {
/* rayrep evstart evstop [disk#] - checks for gates and routes all spectra    */

FILE *f;
char name[16], fpath[80];
char *ev_file[] = {"ev_e", "ev_e3", "ev_tof", "ev_route"}; /* the event files */
int nev_files = 4;	/* the number of event files defined in previous line */
int nvect = 10;		/* the number of vectors to be 'histogramed'          */
unsigned vect[10];      /* the the actual [vector] data to be histogramed     */
int index,end,j,k,g,tag,start,stop;
unsigned i,evstart,evstop,evnum,disk,e,e3,tof,etof,route,e3e,idata,*ptr;
unsigned raw_e,raw_e3,raw_tof,raw_etof,raw_route;
unsigned long e_tof,m_e,m_tof,e3_e,e3_e3e,mass,data,savtim,savdat;

switch (gettoken(name))
{	case T_INTEGER:
		evstart = (unsigned)atoi(name);
		break;
	default:
		fprintf(oual_ounit,"\nERROR(rayrep): Need evstart.");
		flush();
		return(0);
}
switch (gettoken(name)) 
{	case T_INTEGER:
		evstop = (unsigned)atoi(name);
		break;
	default:
		fprintf(oual_ounit,"\nERROR(rayrep): Need evstop.");
		flush();
		return(0);
}
disk = oual_disk;
switch (gettoken(name)) 
{	case T_INTEGER:
		disk = (unsigned)atoi(name);
		break;
}

/* get the events into the event buffers -------------------------------------*/
for (evnum=evstart; evnum<=evstop; evnum++) /* do for each event file group   */
{   for(j=0; j < nev_files; j++) 	    /* do for each type of event file */
    {	index = def_index(ev_file[j]);
	if(index == -1)
	{ 	fprintf(oual_ounit,"\nERROR(rayrep): %s undefined",ev_file[j]);
		flush();
		return(10);
	}
	sprintf(fpath,"data/evt%04d/%s.%03d",disk,ev_file[j],evnum);
	fprintf(oual_ounit,"\nGetting %s from file %s.",ev_file[j],fpath);
	if ((f = fopen (fpath,"r")) == NULL)
	{ 	fprintf(oual_ounit,"\nERROR(rayrep): Cannot open %s",fpath);
		flush();
		return(10);
	}
	fscanf(f," %07u %07u %07u %07u %07lu %07lu",&disk,&tag,&start,&stop,   \
							&savdat,&savtim);
	ptr = oual_define[index].address;
	if (((stop - start) + 1) > oual_define[index].length)
	{ 	fprintf(oual_ounit,"\nERROR(rayrep): event buffer too small.");
	   	flush();
	   	return(10);
	}
	for (i=start; i<=stop; i++)
	{	fscanf(f,"%08u",&idata);
		*ptr++ = idata;
	}
	fclose(f);
    } /* terminates j loop */
/* event buffers now contain event data; now process -------------------------*/

end = getevent(90,0);
if (end == -1) return(2);

fprintf(oual_ounit,"\nProcessing %u events...",end);
for ( i=1; i < end; i++)
{ 	e           = getevent(90,i) & 0xfff;
	e3          = getevent(92,i) & 0xfff;
	tof         = getevent(91,i) & 0xfff;
	oual_router = getevent(93,i) & 0xfff;

	switch (oual_router)
	{	case 0:  route = 0;  break;
		case 1:  route = 10; break;
		case 2:  route = 20; break;
		case 4:  route = 30; break;
		case 8:  route = 40; break;
		case 16: route = 50; break;
		case 32: route = 60; break;
		case 64: route = 70; break;
		case 128: route = 80; break;
		case 256: route = 90; break;
		default: histogram(109,oual_router); route = 100; break;
	}
	if(histogram(route,e) != 0) goto intloop;
	if(histogram(route+1,tof) != 0) goto intloop;

	e_tof=(((((unsigned long)tof & (unsigned long)0xffc)
				<< 10) | (unsigned long)e) >> 2);
	histogram(route+2, (unsigned) e_tof);

	if(histogram(route+3,e3) != 0) goto intloop;
	e3_e=(((((unsigned long)e & (unsigned long)0xfc0)
				<< 6) | (unsigned long)e3) >> 6);
	histogram(route+4, (unsigned) e3_e);

	e3e = (e + e3) / 2;
	if(histogram(route+5,e3e) != 0) goto intloop;

	e3_e3e=(((((unsigned long)e3e & (unsigned long)0xfc0)
				<< 6) | (unsigned long)e3) >> 6);
	histogram(route+6, (unsigned) e3_e3e);

	mass = ((unsigned long)e * ((((unsigned long)oual_t0 - (unsigned long) \
	    	tof)*((unsigned long)oual_t0-(unsigned long)tof)) >> 10)) >> 10;

	mass = mass & 0xfff;
	histogram(route+7, (unsigned) mass);

        m_e=(((((unsigned long)e & (unsigned long)0xffc)
                                << 10) | (unsigned long)mass) >> 2);
        histogram(route+8, (unsigned) m_e);

        m_tof=(((((unsigned long)tof & (unsigned long)0xffc)
                                << 10) | (unsigned long)mass) >> 2);
        histogram(route+9, (unsigned) m_tof);


/*	define the vect array which contains the data so we can use a loop    */
   	vect[0] = e; vect[1] = tof, vect[2] = (unsigned)e_tof; vect[3] = e3; 
	vect[4] = (unsigned)e3_e; vect[5] = e3e; vect[6] = (unsigned)e3_e3e; 
	vect[7]=(unsigned)mass; vect[8]=(unsigned)m_e; vect[9]= (unsigned)m_tof;

	for (g = 1; g <= oual_nrgates; g++)          /* do for each rgate set */
	{	if (oual_rgate[(6*g)+2] % 10 > nvect-1)/*fix this err handling*/
			 printf("\nERROR(rayrep): modulus error.\n");
		else
			data = (unsigned long) vect[oual_rgate[(6*g)+2] % 10];
		if((oual_rgate[6*g] != 0) || (oual_rgate[(6*g)+1] != 0))
		{	for(k=0; k < nvect;k++)
			{ 	if ((route+k == oual_rgate[(6*g)+3]) &&        \
				       (checkrgate(g,oual_rgate[(6*g)+2],data)))
				       histogram(oual_rgate[(6*g)+5],vect[k]);
			}	
		}
	} /* terminates g loop */
	intloop:
} /* terminates i loop */
} /* terminates evnum loop */
return 0;
} /* terminates rayrep main */

